/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * This module originally created by Max d'Ayala (www.dAyala.co.uk)
 *
 * Contributor(s):
 *		Tobias Minich
 *
 * ***** END LICENSE BLOCK ***** */

/* === Global variables === */
var XHR = new Object();
var POSTinfo = new Object();
POSTinfo.isError=true;
POSTinfo.isAbort=false;
POSTinfo.status="0";
POSTinfo.statusText="Undefined";
POSTinfo.msgText="Undefined";
POSTinfo.document=null;
POSTinfo.documentText=null;
POSTinfo.location=null;
POSTinfo.replace=null;

/* === UTILITIES === */

function mD(f,level,msg)
{
  opener.mozileDebug(f,level,msg);
}

/* === WINDOW Events === */
/*
  onload event handler
*/
function PostOnload()
{
	mozileSetVersionById('version');
	POST_HttpRequest( window.arguments[0], window.arguments[1] );
	document.getElementById("abortButton").focus();
	return true;
}

function PostStop()
{
	var	f =	["savepost.js","PostStop"];

  if ( XHR.readyState != 4 ) {
    XHR.abort();
  }
  /* Possibility of locking window open if this try block fails */
	try {
    var n = document.getElementById("saveText2").firstChild;
		n.data = "Save aborted.";
  	n = document.getElementById("acceptButton");
		n.setAttribute( "hidden", "false" );
		n.focus();
  	document.getElementById("abortButton").setAttribute( "hidden", "true" );
  	document.getElementById("saveMeter").setAttribute( "mode", "determined" );
  	document.getElementById("saveMeter").setAttribute( "value", "65" );
	}
	catch(e) {
	  mD(f,1,"Exception: "+e);
	}
	window.arguments[2].isAbort = true;
	mD(f,3,"XMLHttpRequest aborted");
  return true;
}

function PostAccept()
{
  self.close();
	return true;
}

/* === XMLHttpRequest Events === */
function PostComplete()
{
  var	f =	["savepost.js","PostComplete"];

  if ( XHR.readyState == 4 ) {
  	try {
    	document.getElementById("abortButton").setAttribute( "disabled", "true" );
  	} catch (e) {
	    mD(f,1,"Exception: "+e);
  	}
	  POST_Response();
	}

  /*Set up info to be returned to opener.*/
  window.arguments[2].isError = POSTinfo.isError;
  window.arguments[2].status = POSTinfo.status;
  window.arguments[2].statusText = POSTinfo.statusText;
  window.arguments[2].msgText = POSTinfo.msgText;
  if (POSTinfo.document) {
		window.arguments[2].document = POSTinfo.document;
	}
  if (POSTinfo.documentText) {
		window.arguments[2].documentText = POSTinfo.documentText;
	}
	if (POSTinfo.location != null) {
		window.arguments[2].location = POSTinfo.location;
		if (POSTinfo.replace === true)
			window.arguments[2].replace = POSTinfo.replace;
	}
  mD(f,3,"POST completed");
	/*Close window. window.close() doesn't work directly from here.*/
	/*Don't like this but can't get dispatchEvent() to work.*/
	try {
    var b = document.getElementById("acceptButton");
    b.doCommand( PostAccept );
		/*problem handling errors here because window has closed.*/
	} catch (e) {
		mD(f,1,"Exception2: "+e);
	}
	return true;
}
/*
 0 (initial text), 1 and 4 probably won't show, 2 may be too quick as well!
*/
function PostChange()
{
  try {
    var n = document.getElementById("saveText2").firstChild;
    switch ( XHR.readyState )
		{
  	case 1 :
		  n.data = "1 : Initialising...";
			break;
  	case 2 :
		  n.data = "2 : Sending...";
			break;
  	case 3 :
		  n.data = "3 : Receiving acknowledgement...";
			break;
  	case 4 :
		  n.data = "4 : Completed";
			break;
  	}
	}
	catch(e) {
 	  var	f =	["savepost.js","PostChange"];
	  mD(f,0,"Exception: "+e);
	}
	return true;
}

function PostOnerror()
{
  /*What to do here??? Will the onload event still fire to terminate the process???*/
  var	f =	["savepost.js","PostOnerror"];
  mD(f,1,"POST XMLHttpRequest Error");
	return true;
}

/* === FUNCTIONS === */
/*
 Starts an asynchronous request.
*/
function POST_HttpRequest(id, content)
{
  var	f =	["savepost.js","POST_HttpRequest"];
  try {
    XHR = new XMLHttpRequest();
    XHR.open("POST", id.httpSavePath, true, null, null);
		XHR.setRequestHeader( 'Content-Type', id.contentType + "; " + id.characterSet );
		XHR.setRequestHeader( 'Content-Location', id.documentHref );
		XHR.onerror = PostOnerror;
		XHR.onreadystatechange = PostChange;
		XHR.onload = PostComplete;
    XHR.send(content);
		mD(f,3,"POST sent");
  }catch (e){
	  mD(f,0,"Exception: "+e);
  }
}

function POST_Response()
{
  switch ( XHR.status ) {
	case 200 :
	  POST_200();
		break;
	case 204 :
		POST_204();
		break;
	case 201 :
	  POST_201();
		break;
	default :
	  POST_Error( "Unexpected server response" );
	}
	return true;
}

function POST_204() {
  POSTinfo.isError = false;
	POSTinfo.status = "204";
	POSTinfo.statusText = XHR.statusText;
}

function POST_201() {
  POSTinfo.isError = false;
	POSTinfo.status = "201";
	POSTinfo.statusText = XHR.statusText;
}

function POST_200() {
  if ( XHR.responseXML ) {
	  POST_200XML();
  }
  else {
	  POST_Error( "XML response not received from server." );
	}
}

function POST_200XML() {
	var	f =	["savepost.js","POST_200XML"];
  var postNode, psNode, ptNode, plNode, postStatus="", postStatusText="", postLocation=null, postReplace=false;

  try {
  	postNode = XHR.responseXML.getElementsByTagNameNS("http://www.mozile.mozdev.org/ns/save/", "post");
    if (postNode.length > 0) {
      psNode = postNode[0].getElementsByTagName("status");
      if ( (psNode.length > 0) && psNode[0].hasChildNodes() && (psNode[0].firstChild.nodeType == Node.TEXT_NODE) ) {
        postStatus = psNode[0].firstChild.data;
      }
      ptNode = postNode[0].getElementsByTagName("statustext");
      if ( (ptNode.length > 0) && ptNode[0].hasChildNodes() && (ptNode[0].firstChild.nodeType == Node.TEXT_NODE) ) {
        postStatusText = ptNode[0].firstChild.data;
  		}
			plNode = postNode[0].getElementsByTagName("location");
			if (plNode.length > 0) {
				if ( plNode[0].hasChildNodes() && (plNode[0].firstChild.nodeType == Node.TEXT_NODE) ) {
					postLocation = plNode[0].firstChild.data;
				}
				else {
					postLocation = "";
				}
				if ( plNode[0].hasAttributes() ) {
					var plAttr = plNode[0].attributes.getNamedItem("replace");
					if ( plAttr && plAttr.value.toLowerCase() == "true" )
						postReplace = true;
				}

			}
  	}
    if ( (postStatus == "") || (postStatusText == "") ) {
		  var s = "Something is wrong with the XML response."
		  if (postNode.length == 0) {s+="\n<post> tag is missing or namespace is incorrect.\n  Namespace should be;\nhttp://www.mozile.mozdev.org/ns/save/";}
			else if (psNode.length == 0) {s+="\n<status> tag is missing.";}
			else if (postStatus == "") {s+="\nEmpty <status></status> tags.";}
			else if (ptNode.length == 0) {s+="\n<statustext> tag is missing.";}
			else if (postStatusText == "") {s+="\nEmpty <statustext></statustext> tags.";}
 	  	POST_Error( s );
		} else {
      POSTinfo.status = "200";
  		POSTinfo.statusText = XHR.statusText;
			if ( postStatus == "1" ) {
	      POSTinfo.isError = false;
	      POSTinfo.msgText = postStatusText;
	    }
	    else {
	      POSTinfo.isError = true;
	      POSTinfo.msgText = "[" + postStatus + "] " + postStatusText;
	    }
			if ( postLocation != null ) {
				POSTinfo.location = postLocation;
				if ( postReplace )
					POSTinfo.replace = true;
			}
    }
	}
	catch(e) {
		mD(f,2,"Exception: "+e);
		POST_Error( "Exception" );
	}
}
/*
  General error handling function.
	Optional first argument is a short string describing the error.
*/
function POST_Error() {
	var m = "Error";
	if ( arguments[0] ) {
		m += ": " + arguments[0] + "\n";
	}
	if (XHR.status && XHR.responseXML) {
		POSTinfo.status = XHR.status;
		POSTinfo.statusText = XHR.statusText;
		POSTinfo.document = XHR.responseXML;
		var x = parseXMLerror(XHR.responseXML);
		if ( x ) {
			m += "\n" + x;
		}
		POSTinfo.msgText = m;
		if ( XHR.responseText ) {
			POSTinfo.documentText = XHR.responseText;
		}
	}
	else if ( XHR.status && XHR.responseText ) {
		POSTinfo.status = XHR.status;
		POSTinfo.statusText = XHR.statusText;
		POSTinfo.msgText = m;
		POSTinfo.documentText = XHR.responseText;
	}
	else if ( XHR.status ) {
		POSTinfo.status = XHR.status;
		POSTinfo.statusText = XHR.statusText;
		POSTinfo.msgText = m;
	}
	else {
		POSTinfo.status = 999;
		POSTinfo.statusText = "Undefined";
		POSTinfo.msgText = m;
	}
	POSTinfo.isError = true;
}

/*
  Check for parse errors in the XML document.
*/
function parseXMLerror( responseXML ) {
  var	f =	["savepost.js","parseResponseXML"];
  var errtext="";
  
	try {
    // look to see if there is a parse error
    var parserErrorNode = responseXML.getElementsByTagNameNS("http://www.mozilla.org/newlayout/xml/parsererror.xml","parsererror")[0];
  
    if (parserErrorNode) {
      errtext += parserErrorNode.firstChild.data;
      var sourceNode = parserErrorNode.getElementsByTagName("sourcetext")[0];
      if (sourceNode) {
        errtext += "\n" + sourceNode.firstChild.data;
      }
    }
  } catch (e) {
		  mD(f,2,"Exception: "+e);
	}
	return errtext;
}

